# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['tradingview_screener']

package_data = \
{'': ['*']}

install_requires = \
['pandas>=2.0.1,<3.0.0', 'requests>=2.28.2,<3.0.0']

setup_kwargs = {
    'name': 'tradingview-screener',
    'version': '1.0.0',
    'description': 'A package for creating stock screeners with the TradingView API',
    'long_description': "# TradingView-Screener\n\n\n---\n[![PyPi](https://img.shields.io/badge/PyPi-0.3.0-yellow)](https://pypi.org/project/tradingview-screener/)\n[![Downloads](https://pepy.tech/badge/tradingview-screener)](https://pepy.tech/project/tradingview-screener)\n[![Downloads](https://pepy.tech/badge/tradingview-screener/month)](https://pepy.tech/project/tradingview-screener)\n\nYou can get the package directly from [PyPI](https://pypi.org/project/tradezero-api/)\n```\npip install tradingview-screener\n```\n---\n\nThis package allows you to create stock screeners with TradingView's, and retrieve the data directly from the official\nAPI (without doing any kind of web-scraping).\n\n\n#### The following quick-guide will show you how to get started using the tradingview-package module\n\nImport the function and the Enum class\n```python\nfrom tradingview_screener import get_scanner_data, Scanner\n```\n\nAll the scanners available:\n```python\n>>> Scanner.names()\n```\n```\n['premarket_gainers',\n 'premarket_losers',\n 'premarket_most_active',\n 'premarket_gappers',\n 'postmarket_gainers',\n 'postmarket_losers',\n 'postmarket_most_active']\n```\n\nThen, to retrieve the data you must provide the scanner type\n```python\ndf = Scanner.premarket_gainers.get_data()\n```\n\nAnd we get a DataFrame with the data:\n```python\n>>> df\n```\n```\n    name  premarket_change     close    volume  market_cap_basic\n0   VIRI         77.514793    0.6253   4047431      1.146199e+07\n1   CXAI         45.310016    6.2900   1407773      5.398518e+07\n2   BXRX         45.212766    1.8800    469498      4.861120e+06\n3   SPPI         35.264301    0.6905    478063      1.417606e+08\n4   MORF         30.944176   43.5300    239678      1.720921e+09\n5   PTPI         29.166667    3.8400    828932      8.020600e+06\n6   ARWR         21.918721   30.0200    630642      3.251471e+09\n7   EDUC         21.076233    2.2300     44927      1.943064e+07\n8   ASTS         20.183486    4.3600   1814082      8.723593e+08\n9   ZFOX         18.348624    1.0900    460906      1.292519e+08\n10  MEDP         16.511802  187.2600    598436      5.814453e+09\n...\n\n[50 rows x 5 columns]\n```\n\nIf you aren't yet familiar with Pandas DataFrames, you can convert the output to a list of dictionaries like so:\n```python\n>>> df.to_dict('records')\n```\n```\n[\n    {'name': 'VIRI', 'premarket_change': 77.5147929, 'close': 0.6253, 'volume': 4047431, 'market_cap_basic': 11461993.0},\n    {'name': 'CXAI', 'premarket_change': 45.3100159, 'close': 6.29, 'volume': 1407773, 'market_cap_basic': 53985175.00000001},\n    {'name': 'BXRX', 'premarket_change': 45.21276596, 'close': 1.88, 'volume': 469498, 'market_cap_basic': 4861120.0},\n    {'name': 'SPPI', 'premarket_change': 35.26430123, 'close': 0.6905, 'volume': 478063, 'market_cap_basic': 141760626.0},\n    {'name': 'MORF', 'premarket_change': 30.94417643, 'close': 43.53, 'volume': 239678, 'market_cap_basic': 1720920967.0},\n    ...\n]\n```\n\nOr to get the most active during the pre-market session:\n```python\n>>> Scanner.premarket_most_active.get_data()\n```\n```\n    name  premarket_volume     close     volume  market_cap_basic\n0   MULN          27981441    0.0960  756912573      3.640486e+08\n1   VIRI          16779935    0.6253    4047431      1.146199e+07\n2   BBBY          12464514    0.1888  539779156      8.082502e+07\n3   SPPI           8098728    0.6905     478063      1.417606e+08\n4   ZFOX           5510783    1.0900     460906      1.292519e+08\n5    FRC           5165413   16.0000   90150729      2.928000e+09\n6   BXRX           4524110    1.8800     469498      4.861120e+06\n7   XELA           3525650    0.0397   49643555      5.058590e+07\n8   CXAI           2464392    6.2900    1407773      5.398518e+07\n9   IDEX           2250652    0.0399   67497209      2.787359e+07\n10  AGFY           2250428    0.2123    5197584      1.884452e+06\n...\n\n[50 rows x 5 columns]\n```\n\nOnce you have the data you can filter the results however you want.  \nIn our case we only want stocks that have a pre-market change above 10% and a price bigger than $3 \n```python\n>>> filt = (df['premarket_change'] > 10) & (df['close'] > 3)\n>>> df[filt]\n```\n```\n    name  premarket_change    close   volume  market_cap_basic\n1   CXAI         45.310016    6.290  1407773      5.398518e+07\n4   MORF         30.944176   43.530   239678      1.720921e+09\n5   PTPI         29.166667    3.840   828932      8.020600e+06\n6   ARWR         21.918721   30.020   630642      3.251471e+09\n8   ASTS         20.183486    4.360  1814082      8.723593e+08\n10  MEDP         16.511802  187.260   598436      5.814453e+09\n15  GDHG         12.159612    3.709     3492      1.854500e+08\n16   PCT         11.368015    5.190  1294479      8.494532e+08\n19  APLM         10.643564    4.040   159000      2.590733e+07\n\n[9 rows x 5 columns]\n```\n\n-----  \nYou can also override the settings that are being passed to the API.\n\nFor example if you want to get the first 200 results instead of just 50:\n```python\n>>> Scanner.premarket_losers.get_data(range=[0, 200])\n```\n```\n     name  premarket_change    close    volume  market_cap_basic\n0    TCON        -51.123596   1.7800     18817      4.240412e+07\n1    VYNT        -31.709091   0.5500    194750      3.447239e+06\n2    NCPL        -27.626818   1.5199   6111277      9.228493e+06\n3     FRC        -25.937500  16.0000  90150729      2.928000e+09\n4     SFR        -25.500000   2.0000  16709427      9.651445e+07\n..    ...               ...      ...       ...               ...\n195  RBOT         -2.643172   2.2700    123525      2.859286e+08\n196  HPCO         -2.640264   0.6060    151701      1.416420e+07\n197  BLDP         -2.608696   4.6000   1776254      1.372613e+09\n198   TRX         -2.601535   0.5343    205843      1.479665e+08\n199   SOI         -2.573529   8.1600    185698      3.803123e+08\n\n[200 rows x 5 columns]\n```\n\nYou can also specify the columns you want to get\n```python\n>>> Scanner.premarket_gainers.get_data(range=[0, 10], columns=['name', 'close', 'premarket_change', 'logoid', 'description', 'type', 'VWAP', 'MACD.macd'])\n```\n```\n   name    close  premarket_change  ...   type       VWAP MACD.macd\n0  VIRI   0.6253         91.907884  ...  stock   0.644767  0.086156\n1  BXRX   1.8800         43.617021  ...  stock   2.018500  0.082743\n2  CXAI   6.2900         39.745628  ...  stock   7.100400  1.106645\n3  SPPI   0.6905         34.047791  ...  stock   0.696000 -0.018506\n4  MORF  43.5300         30.484723  ...  stock  43.910000  1.133547\n5  ASTS   4.3600         25.688073  ...  stock   4.308333 -0.396030\n6  ARWR  30.0200         21.585610  ...  stock  30.348067  0.864783\n7  PTPI   3.8400         21.093750  ...  stock   4.450000  0.941922\n8  EDUC   2.2300         21.076233  ...  stock   2.280000 -0.257153\n9  AGFY   0.2123         17.192652  ...  stock   0.217233 -0.016040\n\n[10 rows x 8 columns]\n```\n\nFor the full list of columns have a look at the following dictionary (there are more that will be added in the future):\n```python\n>>> from tradingview_screener.screener import COLUMNS\n>>> COLUMNS\n```\n```\n{'1-Month High': 'High.1M',\n '1-Month Low': 'Low.1M',\n '1-Year Beta': 'beta_1_year',\n '3-Month High': 'High.3M',\n '3-Month Low': 'Low.3M',\n '3-Month Performance': 'Perf.3M',\n '52 Week High': 'price_52_week_high',\n '52 Week Low': 'price_52_week_low',\n '5Y Performance': 'Perf.5Y',\n '6-Month High': 'High.6M',\n '6-Month Low': 'Low.6M',\n '6-Month Performance': 'Perf.6M',\n 'All Time High': 'High.All',\n 'All Time Low': 'Low.All',\n 'All Time Performance': 'Perf.All',\n 'Aroon Down (14)': 'Aroon.Down',\n 'Aroon Up (14)': 'Aroon.Up',\n ...}\n```\n\n\n## Creating Custom Stock Screeners\n\n```python\nfrom tradingview_screener import Query, Column\n```\n\n\nCreate a query (like you would in a SQL database):\n```python\nq = Query().select('name', 'close', 'volume', 'relative_volume_10d_calc', 'market_cap_basic')\n```\n\nAnd to get the results we can call `get_scanner_data()` which will return a tuple with the number of results `COUNT(*)`\nthat matched our query, and the actual data (as a pandas DataFrame):\n```python\nnum_rows, df = q.get_scanner_data()\nprint('Number of rows:', num_rows)\n```\n```\nNumber of rows: 5271\n```\n\nAnd the data:\n```python\ndf\n```\n```\n    name      close   volume  relative_volume_10d_calc  market_cap_basic\n0   SASI   0.421101     2176                  0.122958      4.536400e+06\n1   BROG   5.050000      491                  0.170126      4.445781e+08\n2   LAZR   6.020000  4643780                  0.644816      2.222977e+09\n3    PIK   0.598500    88926                  0.682655      4.601384e+06\n4   HAYW  12.040000  1464436                  0.986823      2.559983e+09\n..   ...        ...      ...                       ...               ...\n45  ATIP   0.271400   243505                  1.096411      5.628409e+07\n46  CLFD  43.680000   242512                  0.736585      6.647937e+08\n47  CRCT   9.130000    66012                  1.154169      2.003323e+09\n48   ENZ   2.530000   118980                  0.529591      1.256469e+08\n49  WLGS   2.220000  5589780                       NaN               NaN\n\n[50 rows x 5 columns]\n```\n\nAs you may have noticed, there are only 50 rows in our dataframe, while `num_rows` is 5271,  \nthis is because by default there is a `LIMIT` of 50, although you can override that by providing your own.\nBut, note that the more results you try to fetch, the more work the server needs to do, and therefore the network request\nwill take more time, so It's something to keep in mind.\n\nYou can also filter the results:\n```python\nq = Query().select('name', 'close', 'volume', 'relative_volume_10d_calc')\\\n    .where(\n        Column('market_cap_basic').between(1_000_000, 50_000_000), \n        Column('relative_volume_10d_calc') > 1.2, \n        Column('MACD.macd') >= 'MACD.signal'\n    )\nq.get_scanner_data()\n```\n```\n(493,\n      name    close  volume  relative_volume_10d_calc\n 0    CMPD  2.90000    1308                  1.806380\n 1   ABNAF  0.08500    7210                  1.365401\n 2    TARA  3.16000  102277                  1.697113\n 3   FLYLF  0.74853   15000                  1.973632\n 4   OGBLY  0.04000   45005                  6.980766\n ..    ...      ...     ...                       ...\n 45   LPCN  0.26000  348622                  2.326042\n 46   ELTP  0.03300  561900                  2.117304\n 47   FNAM  0.70000    2000                  1.350348\n 48     UK  1.04000  315203                  1.409103\n 49   VERO  0.18880  218379                  1.435621\n \n [50 rows x 4 columns])\n```\n\nTo order the results:\n```python\nq = Query().select('name', 'close', 'volume', 'relative_volume_10d_calc')\\\n    .where(\n        Column('market_cap_basic').between(1_000_000, 50_000_000), \n        Column('relative_volume_10d_calc') > 1.2, \n        Column('MACD.macd') >= 'MACD.signal'\n    )\\\n    .order_by('volume', ascending=False)\nq.get_scanner_data()\n```\n```\n(493,\n     name    close    volume  relative_volume_10d_calc\n 0    DXF  0.36990  92255915                 56.544501\n 1   DPLS  0.00555  51685577                  1.528578\n 2   LGHL  0.26800  48563857                 70.600598\n 3   JZXN  0.26000  45771791                 82.053268\n 4   PVSP  0.00060  45491685                  1.786331\n ..   ...      ...       ...                       ...\n 45  RMSL  0.01290   1943387                  1.542694\n 46  TIRX  1.63000   1937953                 43.849456\n 47  TNXP  0.52000   1870167                  2.578986\n 48    MF  1.30000   1837216                108.688496\n 49  EWRC  0.00280   1761115                 11.767738\n \n [50 rows x 4 columns])\n```\n\nTo get the slice `[5:15]`:\n```python\nq = Query().select('name', 'close', 'volume', 'relative_volume_10d_calc')\\\n    .where(\n        Column('market_cap_basic').between(1_000_000, 50_000_000), \n        Column('relative_volume_10d_calc') > 1.2, \n        Column('MACD.macd') >= 'MACD.signal'\n    )\\\n    .order_by('volume', ascending=False)\\\n    .offset(5)\\\n    .limit(15)\nq.get_scanner_data()\n```\n```\n(493,\n    name    close    volume  relative_volume_10d_calc\n 0  INTK  0.00050  32999277                  3.749458\n 1  BIEL  0.00040  26141639                  1.350728\n 2  GSUN  1.35000  24240204                 98.543864\n 3  FTXP  0.00020  21113909                  1.894780\n 4  IGEX  0.00080  20660879                  1.211745\n 5  HPNN  0.00090  16051557                  2.295840\n 6  SGMD  0.00020  15761527                  2.232289\n 7  ERBB  0.00102  14257745                  1.805648\n 8  ASTA  0.00220  13801619                  1.537809\n 9  BANT  0.00020  13514923                  1.573909)\n```\n\n### Reusing Scanners\n\nTo avoid rewriting the same query again and again, you can save the query to a variable and just call \n`get_scanner_data()` again and again to get the latest data:\n```python\ntop_50_bullish = Query().select('name', 'close', 'volume', 'relative_volume_10d_calc')\\\n    .where(\n        Column('market_cap_basic').between(1_000_000, 50_000_000), \n        Column('relative_volume_10d_calc') > 1.2, \n        Column('MACD.macd') >= 'MACD.signal'\n    )\\\n    .order_by('volume', ascending=False)\\\n    .limit(50)\n```\n\nAnd you can just call it like so:\n```python\ntop_50_bullish.get_scanner_data()\n```\n```\n(493,\n     name    close    volume  relative_volume_10d_calc\n 0    DXF  0.36990  92255915                 56.544501\n 1   DPLS  0.00555  51685577                  1.528578\n 2   LGHL  0.26800  48563857                 70.600598\n 3   JZXN  0.26000  45771791                 82.053268\n 4   PVSP  0.00060  45491685                  1.786331\n ..   ...      ...       ...                       ...\n 45  RMSL  0.01290   1943387                  1.542694\n 46  TIRX  1.63000   1937953                 43.849456\n 47  TNXP  0.52000   1870167                  2.578986\n 48    MF  1.30000   1837216                108.688496\n 49  EWRC  0.00280   1761115                 11.767738\n \n [50 rows x 4 columns])\n```\n\n\n```python\ntop_50_bullish.get_scanner_data()\n```\n```\n(493,\n     name    close    volume  relative_volume_10d_calc\n 0    DXF  0.36990  92255915                 56.544501\n 1   DPLS  0.00555  51685577                  1.528578\n 2   LGHL  0.26800  48563857                 70.600598\n 3   JZXN  0.26000  45771791                 82.053268\n 4   PVSP  0.00060  45491685                  1.786331\n ..   ...      ...       ...                       ...\n 45  RMSL  0.01290   1943387                  1.542694\n 46  TIRX  1.63000   1937953                 43.849456\n 47  TNXP  0.52000   1870167                  2.578986\n 48    MF  1.30000   1837216                108.688496\n 49  EWRC  0.00280   1761115                 11.767738\n \n [50 rows x 4 columns])\n```\n\n\n---\n\n## Getting All The Stock Symbols\n\n#### There is a function that allows you to get all the symbols from a given exchange.\n\n\nImport the function\n```python\nfrom tradingview_screener import get_all_symbols\n```\n\nGet all the symbols in the US stock market:\n```python\nall_symbols = get_all_symbols()\n```\n\n```python\nlen(all_symbols)\n```\n```\n18451\n```\n\nThe first 10:\n```python\nall_symbols[:10]\n```\n```\n['IIGD', 'TBGPF', 'HLAN', 'FVC', 'GLNG', 'MMC', 'PFHO', 'IDU', 'AMLH', 'IHT']\n```\n\nYou can also filter them by exchange, valid exchanges include: `'AMEX', 'OTC', 'NYSE', 'NASDAQ'`\n```python\nsymbols = get_all_symbols(exchanges={'NYSE', 'NASDAQ'})\nlen(symbols), symbols[:10]\n```\n```\n(7633,\n ['TYRA',\n  'ACRS',\n  'RDNT',\n  'DCI',\n  'LIBY',\n  'EGIO',\n  'AGIO',\n  'KRON',\n  'BRO',\n  'PINC'])\n```\n\nGet all OTC stocks:\n```python\nsymbols = get_all_symbols(exchanges={'OTC'})\nlen(symbols), symbols[:10]\n```\n```\n(7933,\n ['ADMQ',\n  'CSTPF',\n  'LSMG',\n  'LMGIF',\n  'PPMT',\n  'JBFCF',\n  'CVSI',\n  'RXLSF',\n  'STBFY',\n  'GNZUF'])\n```\n",
    'author': 'shner-elmo',
    'author_email': '770elmo@gmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/shner-elmo/TradingView-Screener',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.10,<4.0',
}


setup(**setup_kwargs)
